import click
import os

from kfk.command import kfk
from kfk.kubectl_command_builder import Kubectl
from kfk import topics_command
from kfk import users_command
from kfk.commons import print_missing_options_for_command
from kfk.constants import *


@click.option('-n', '--namespace', help='Namespace to use', required=True)
@click.option('-c', '--cluster', help='Cluster to use', required=True)
@click.option('--native', help='List configs for the given entity natively.', is_flag=True)
@click.option('--describe', help='List configs for the given entity.', is_flag=True)
@click.option('--delete-config', help='Config keys to remove', multiple=True)
@click.option('--add-config', help='Key Value pairs of configs to add.', multiple=True)
@click.option('--entity-name', help='Name of entity', required=True)
@click.option('--entity-type', help='Type of entity (topics/users/clusters)',
              type=click.Choice(['topics', 'users', 'clusters'], case_sensitive=True))
@kfk.command()
def configs(entity_type, entity_name, add_config, delete_config, describe, native, cluster, namespace):
    """Add/Remove entity config for a topic, client, user or cluster"""
    if len(add_config) > 0 or len(delete_config) > 0:
        if entity_type == "topics":
            topics_command.alter(entity_name, None, None, add_config, delete_config, cluster, namespace)
        elif entity_type == "users":
            users_command.alter(entity_name, None, None, False, False, tuple(), None, None, None, None, None,
                                add_config, delete_config, cluster, namespace)
        elif entity_type == "clusters":
            click.echo("Not implemented")
    elif describe:
        if entity_type == "topics":
            if native:
                native_command = "bin/kafka-configs.sh --bootstrap-server {cluster}-kafka-brokers:{port} --entity-type " \
                                 "{entity_type} --entity-name {entity_name} --describe"
                os.system(
                    Kubectl().exec("-it", cluster + "-kafka-0").container("kafka").namespace(namespace).exec_command(
                        native_command).build().format(cluster=cluster, port=KAFKA_PORT, entity_type=entity_type,
                                                       entity_name=entity_name))
            else:
                topics_command.describe(entity_name, None, False, None, cluster, namespace)
        elif entity_type == "users":
            click.echo("Not implemented")
        elif entity_type == "clusters":
            click.echo("Not implemented")

    else:
        print_missing_options_for_command("configs")



import click
import pkg_resources

from kfk.command import kfk
from kfk.config import KUBECTL_VERSION, STRIMZI_VERSION


@kfk.command()
def version():
    """Prints the version of Strimzi Kafka CLI"""
    version = pkg_resources.require("strimzi-kafka-cli")[0].version
    click.echo("CLI Version: " + version)
    click.echo("Strimzi Version: " + STRIMZI_VERSION)
    click.echo("Kubectl Version: " + KUBECTL_VERSION)



from kfk.command import kfk
from kfk.clusters_command import clusters
from kfk.topics_command import topics
from kfk.users_command import users
from kfk.console_command import console_consumer, console_producer
from kfk.configs_command import configs
from kfk.version_command import version
from kfk.acls_command import acls
from kfk.setup import setup


setup()


import click
import os

from kfk.command import kfk
from kfk.option_extensions import NotRequiredIf, RequiredIf
from kfk.kubectl_command_builder import Kubectl
from kfk.commons import print_missing_options_for_command
from kfk import users_command

PRINCIPAL_SPLIT_CHAR = ":"


@click.option('-n', '--namespace', help='Namespace to use.', required=True)
@click.option('-c', '--kafka-cluster', help='Cluster to use.', required=True)
@click.option('--remove', help='Indicates you are trying to remove ACLs.', is_flag=True)
@click.option('--resource-pattern-type',
              help="The type of the resource pattern or <ANY|MATCH|LITERAL|PREFIXED> pattern filter. When adding "
                   "acls, this should be a specific pattern type, e.g. 'literal' or 'prefixed'. (default: literal)",
              default='literal')
@click.option('--deny-host', help='Host which User will not have access. (default: *)', default='*')
@click.option('--allow-host', help='Host which User will have access. (default: *)', default='*')
@click.option('--operation', 'operation_tuple', help='Operation that is being allowed or denied. (default: All)',
              default=["All"], multiple=True)
@click.option('--deny-principal',
              help='principal is in principalType:name format. By default anyone not added through --allow-principal '
                   'is denied access. You only need to use this option as negation to already allowed set.',
              required=True, cls=NotRequiredIf, not_required_if=['list', 'allow_principal'])
@click.option('--allow-principal',
              help='principal is in principalType:name principal format. Note that principalType must be supported '
                   'by the Authorizer being used.', required=True, cls=NotRequiredIf,
              not_required_if=['list', 'deny_principal'])
@click.option('--add', help='Indicates you are trying to add ACLs.', is_flag=True)
@click.option('--group', help='Consumer Group ACLs.')
@click.option('--cluster', help='Cluster ACLs.')
@click.option('--topic', help='Topic ACLs.')
@click.option('--list',
              help='List ACLs for the specified resource, use --topic <topic> or --group <group> or --cluster to '
                   'specify a resource.',
              is_flag=True)
@kfk.command()
def acls(list, topic, cluster, group, add, allow_principal, deny_principal, operation_tuple, allow_host, deny_host,
         resource_pattern_type, remove, kafka_cluster, namespace):
    """This tool helps to manage ACLs on Kafka."""
    if list:
        native_command = "bin/kafka-acls.sh --authorizer-properties zookeeper.connect=localhost:12181 --list {topic}" \
                         "{cluster} {group}"
        os.system(
            Kubectl().exec("-it", "{kafka_cluster}-zookeeper-0").container("zookeeper").namespace(
                namespace).exec_command(
                native_command).build().format(kafka_cluster=kafka_cluster, topic=(topic and '--topic ' + topic or ''),
                                               cluster=(cluster and '--cluster ' + cluster or ''),
                                               group=(group and '--group ' + group or '')))
    elif add or remove:
        alter(topic, cluster, group, add, remove, allow_principal, deny_principal, operation_tuple, allow_host, deny_host,
              resource_pattern_type, kafka_cluster, namespace)
    else:
        print_missing_options_for_command("acls")


def alter(topic, cluster, group, add, remove, allow_principal, deny_principal, operation_tuple, allow_host, deny_host,
          resource_pattern_type, kafka_cluster, namespace):
    resource_type_dict = get_resource_type_dict(topic, cluster, group)

    if allow_principal:
        type = "allow"
        # TODO: click exception here
        allow_principal_arr = allow_principal.split(PRINCIPAL_SPLIT_CHAR)
        principal_type = allow_principal_arr[0]
        principal_name = allow_principal_arr[1]
        host = allow_host
    else:
        type = "deny"
        # TODO: click exception here
        deny_principal_arr = deny_principal.split(PRINCIPAL_SPLIT_CHAR)
        principal_type = deny_principal_arr[0]
        principal_name = deny_principal_arr[1]
        host = deny_host
    if principal_type == "User":
        for resource_type, resource_name in resource_type_dict.items():
            users_command.alter(principal_name, None, None, add, remove, operation_tuple, host, type,
                                resource_type, resource_name, resource_pattern_type, tuple(), tuple(),
                                kafka_cluster, namespace)


def get_resource_type_dict(topic, cluster, group):
    resource_type_dict = {}

    if topic is not None:
        resource_type_dict["topic"] = topic
    if cluster is not None:
        resource_type_dict["cluster"] = cluster
    if group is not None:
        resource_type_dict["group"] = group

    return resource_type_dict



import os
import io
import tempfile
import ntpath

from kfk.kubectl_command_builder import Kubectl
from kfk.utils import convert_string_to_type
from kfk.constants import *


def print_missing_options_for_command(command_str):
    print("Missing options: kfk {command_str} [OPTIONS] \nTry 'kfk {command_str} --help' for help.".format(
        command_str=command_str))


def print_resource_found_msg(cluster, namespace):
    print("No resource found in Kafka cluster: {cluster}, namespace: {namespace}".format(
        cluster=cluster, namespace=namespace))


def delete_last_applied_configuration(resource_dict):
    if "annotations" in resource_dict["metadata"]:
        del resource_dict["metadata"]["annotations"]["kubectl.kubernetes.io/last-applied-configuration"]


def add_resource_kv_config(config, dict_part, *converters):
    if type(config) is tuple:
        for config_str in config:
            for converter in converters:
                config_str = converter(config_str)
            config_arr = get_kv_config_arr(config_str)
            dict_part[config_arr[0]] = convert_string_to_type(config_arr[1])
    else:
        config_arr = get_kv_config_arr(config)
        dict_part[config_arr[0]] = convert_string_to_type(config_arr[1])


def get_kv_config_arr(config_str):
    # TODO: exception here
    return config_str.split('=')


def delete_resource_config(config, dict_part, *converters):
    if type(config) is tuple:
        for config_str in config:
            for converter in converters:
                config_str = converter(config_str)
            if config_str in dict_part:
                del dict_part[config_str]
    else:
        del dict_part[config]


def resource_exists(resource_type, resource_name, cluster, namespace):
    return resource_name in os.popen(
        Kubectl().get().resource(resource_type).label("strimzi.io/cluster={cluster}").namespace(
            namespace).build().format(
            cluster=cluster)).read()


def get_resource_yaml(resource_type, resource_name, namespace):
    return os.popen(
        Kubectl().get().resource(resource_type, resource_name).namespace(namespace).output("yaml").build()).read()


def get_resource_as_file(resource_type, resource_name, namespace):
    topic_yaml = get_resource_yaml(resource_type, resource_name, namespace)
    return io.StringIO(topic_yaml)


def create_temp_file(content):
    temp_file = tempfile.NamedTemporaryFile(mode='w+')
    temp_file.write(content)
    temp_file.flush()
    return temp_file


def transfer_file_to_container(source_file_path, dest_file_path, container, pod, namespace):
    os.system(Kubectl().cp(source_file_path, "{namespace}/{pod}:" + dest_file_path).container(container).build().format(
        namespace=namespace, pod=pod))


def apply_client_config_from_file(native_command, config_file_path, config_file_flag, container, pod, namespace):
    port = KAFKA_PORT
    delete_file_command = ""
    with open(config_file_path) as file:
        data = file.read()
        temp_file = create_temp_file(data)
        lines = []
        with open(temp_file.name) as temp_file:
            for cnt, producer_property in enumerate(temp_file):
                producer_property = producer_property.strip()
                if "security.protocol" in producer_property:
                    producer_property_arr = get_kv_config_arr(producer_property)
                    if producer_property_arr[1] == KAFKA_SSL:
                        port = KAFKA_SECURE_PORT
                if "ssl.truststore.location" in producer_property or "ssl.keystore.location" in producer_property:
                    producer_property_arr = get_kv_config_arr(producer_property)
                    file_path = producer_property_arr[1]
                    new_file_path = "/tmp/" + ntpath.basename(file_path)
                    transfer_file_to_container(file_path, new_file_path, container, pod, namespace)
                    producer_property = producer_property_arr[0] + "=" + new_file_path
                    delete_file_command = delete_file_command + "rm -rf" + SPACE + new_file_path + SEMICOLON
                lines.append(producer_property)
        with open(temp_file.name, 'w') as temp_file:
            for line in lines:
                temp_file.write(line + NEW_LINE)
        new_config_file_path = "/tmp/" + ntpath.basename(config_file_path)
        transfer_file_to_container(temp_file.name, new_config_file_path, container, pod, namespace)
        delete_file_command = delete_file_command + "rm -rf" + SPACE + new_config_file_path + SEMICOLON
        native_command = native_command + SPACE + config_file_flag + SPACE + new_config_file_path
        temp_file.close()
    return native_command.format_map(SafeDict(port=port)) + SEMICOLON + delete_file_command


class SafeDict(dict):
    def __missing__(self, key):
        return '{' + key + '}'



import click


class NotRequiredIf(click.Option):
    def __init__(self, *args, **kwargs):
        self.not_required_if = kwargs.pop('not_required_if')
        assert self.not_required_if, "'not_required_if' parameter required"
        kwargs['help'] = (kwargs.get('help', '') +
                          ' This argument is mutually exclusive with %s' %
                          self.not_required_if
                          ).strip()
        super(NotRequiredIf, self).__init__(*args, **kwargs)

    def handle_parse_result(self, ctx, opts, args):
        we_are_present = self.name in opts

        for not_required_if in self.not_required_if:
            other_present = not_required_if in opts
            if other_present is True:
                break

        if other_present:
            if we_are_present:
                raise click.UsageError(
                    "Illegal usage: `%s` is mutually exclusive with `%s`" % (
                        self.name, self.not_required_if))
            else:
                self.required = None

        return super(NotRequiredIf, self).handle_parse_result(
            ctx, opts, args)


class RequiredIf(click.Option):
    def __init__(self, *args, **kwargs):
        self.required_if = kwargs.pop('required_if')
        assert self.required_if, "'required_if' parameter required"
        kwargs['help'] = (kwargs.get('help', '') +
                          ' This argument is mutually inclusive with %s' %
                          self.required_if
                          ).strip()
        super(RequiredIf, self).__init__(*args, **kwargs)

    def handle_parse_result(self, ctx, opts, args):
        we_are_present = self.name in opts
        for required_if in self.required_if:
            other_present = required_if in opts
            if other_present is True:
                break

        if other_present or we_are_present:
            self.required = True
        else:
            self.required = None

        return super(RequiredIf, self).handle_parse_result(
            ctx, opts, args)



import click
import os

from kfk.command import kfk
from kfk.option_extensions import NotRequiredIf
from kfk.commons import print_missing_options_for_command
from kfk.kubectl_command_builder import Kubectl


@click.option('-n', '--namespace', help='Namespace to use')
@click.option('-o', '--output',
              help='Output format. One of: json|yaml|name|go-template|go-template-file|template|templatefile|jsonpath'
                   '|jsonpath-file.')
@click.option('--describe', help='List details for the given cluster.', is_flag=True)
@click.option('--list', help='List all available clusters.', required=True, is_flag=True)
@click.option('--cluster', help='Cluster Name', required=True, cls=NotRequiredIf, not_required_if=['list'])
@kfk.command()
def clusters(cluster, list, describe, output, namespace):
    """The kafka cluster(s) to be created, altered or described. """
    if list:
        os.system(Kubectl().get().kafkas().namespace(namespace).build())
    elif describe:
        if output is not None:
            os.system(
                Kubectl().get().kafkas(cluster).namespace(namespace).output(output).build())
        else:
            os.system(
                Kubectl().describe().kafkas(cluster).namespace(namespace).build())
    else:
        print_missing_options_for_command("clusters")



from setuptools import setup

with open("README.md", "r") as file:
    long_description = file.read()

setup(
    name='strimzi-kafka-cli',
    version='0.1.0-alpha28',
    description="Command Line Interface for Strimzi Kafka Operator",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=['kfk'],
    package_dir={'kfk': 'kfk'},
    install_requires=[
        'Click==7.1.2',
        'PyYAML==5.3.1',
        'wget==3.2',
    ],
    entry_points='''
        [console_scripts]
        kfk=kfk.main:kfk
    ''',

    url="https://github.com/systemcraftsman/strimzi-kafka-cli",
    python_requires='>=3.5',
)
